#include <Wire.h>
#include <Adafruit_RGBLCDShield.h>
#include <utility/Adafruit_MCP23017.h>

Adafruit_RGBLCDShield lcd = Adafruit_RGBLCDShield();

#define OFF 0
#define ON 0x1

int currentmode = 0;    //mode of editing
                        //0 - duration
                        //1 - aftertime
                        //2 - times
int duration = 15;      //duration of flash in 1/10s (0.5+)
int aftertime = 10;     //duration of post-flash highligh in 1/10s (0+)
int times = 5;          //number of repetitions

//display data in config mode
void display() 
{
  lcd.clear();
  lcd.setCursor(0,0);
  if (currentmode == 0)
  {
    lcd.print("Flash: ");
    lcd.print(duration / 10);
    lcd.print(".");
    lcd.print(duration % 10);
    lcd.print("s");
  }
  else if (currentmode == 1)
  {
      lcd.print("Check: ");
      lcd.print(aftertime / 10);
      lcd.print(".");
      lcd.print(aftertime % 10);
      lcd.print("s");
  }
  else if (currentmode == 2)
  {
      lcd.print("Times: ");
      lcd.print(times);
  }
}

//application entry point
void setup() {
  Serial.begin(9600);
  pinMode(13, OUTPUT);    //power for relay
  pinMode(12, OUTPUT);    //signal for relay, high is open, low is closed
  digitalWrite(13, HIGH);
  digitalWrite(12, HIGH);
  lcd.begin(16, 2);
  lcd.setBacklight(ON);
  display();  
}

//
bool delayDecisecond(int decisecond)
{
  int i, j;

  unsigned long torun = decisecond * 100;
  unsigned long toend = millis() + torun;

  while (millis() < toend)
  {
    uint8_t buttons = lcd.readButtons();
    if (buttons & BUTTON_SELECT)
    {
      while (lcd.readButtons() & BUTTON_SELECT);
      return true;          
    }
  }
  return false;
}

void displayAtRun(int attempt, const char * mode) 
{
  lcd.clear();
  lcd.setCursor(0,0);
  lcd.print("Exercise ");
  lcd.print(attempt);
  lcd.setCursor(0,1);
  lcd.print(mode);
}

bool workCycle(int attempt)
{
  bool rc = false;

  displayAtRun(attempt, "wait");

  if (delayDecisecond(20 + random(0, 20)))
    return true;

  displayAtRun(attempt, "flash");
  digitalWrite(12, LOW);
  if (delayDecisecond(duration))
    rc = true;
  digitalWrite(12, HIGH);

  if (!rc && aftertime > 0)
  {
    displayAtRun(attempt, "check");
    if (!delayDecisecond(20))
    {
      digitalWrite(12, LOW);  
      if (delayDecisecond(aftertime))
        rc = true;
      digitalWrite(12, HIGH);
    }
    else
      rc = true;
  }
  return rc;
}

void loop() {
    uint8_t buttons = lcd.readButtons();
    if (buttons & BUTTON_UP) 
    {
      currentmode++;
      if (currentmode == 3)
        currentmode = 0;
      display();
      while (lcd.readButtons() & BUTTON_UP);
    }
    else if (buttons & BUTTON_DOWN) 
    {
      display();
      while (lcd.readButtons() & BUTTON_DOWN);
    }
    else if (buttons & BUTTON_LEFT) 
    {
      if (currentmode == 0)
      {
          if (duration > 5)
            duration--;
      }
      else if (currentmode == 1)
      {
          if (aftertime > 0)
            aftertime--;
      }
      else if (currentmode == 2)
      {
          if (times > 1)
            times--;
      }
      display();
      while (lcd.readButtons() & BUTTON_LEFT);
    }
    else if (buttons & BUTTON_RIGHT) 
    {
      if (currentmode == 0)
      {
          if (duration < 50)
            duration++;
      }
      else if (currentmode == 1)
      {
          if (aftertime < 50)
            aftertime++;
      }
      else if (currentmode == 2)
      {
          if (times < 20)
            times++;
      }
      display();
      while (lcd.readButtons() & BUTTON_RIGHT);
    }
    else if (buttons & BUTTON_SELECT) 
    {
      randomSeed(micros() % 4096);
      while (lcd.readButtons() & BUTTON_SELECT);
      lcd.setBacklight(OFF);
      int i;
      for (i = 1; i <= times; i++)
        if (workCycle(i))
          break;
      display();          
      lcd.setBacklight(ON);
    }
}


